#!/usr/bin/perl -w

# name of the program: cut_vector.pl

# INPUT: 	xxx.tag (fasta file with vectors/adapters)
# 		yyy.fas fasta file to be cleaned and sorted out.

# OUPUT: yyy_m8.tbl results of blast againts te adapter file
#		yyy_tagY.fas 1 fasta file with all of the cleaned sequences that had the given tag
#		yyy_NOTAG.fas 1 fasta file with all of the cleaned sequences that did not have tag


  use warnings;
  use strict;
  use subprogramQDD;

my $syst = '';
my $fas_file = ''; # file wo path
my $fas_folder = ''; # path to fas_file
my $outfolder = '';
my $blast_path = '';
my $blast_version = '';
my $Evector = '';
my $adapt_file = ''; # file wo path
#my $adapt_folder = ''; # path to tag_file
# if $noadapt ==1 writes one fasta file with sequences that had adapter at the begining, and another if they did not. 
# if $noadapt ==0 write one fasta file with all sequence (including noadapter sequences)
my $noadapt = 1;

if ( scalar@ARGV == 9 )
{
   	$syst = $ARGV[0];
   	$fas_file = $ARGV[1];
   	$fas_folder = $ARGV[2];
   	$outfolder = $ARGV[3];
   	$blast_path = $ARGV[4];
   	$blast_version = $ARGV[5];
   	$Evector = $ARGV[6];
   	$adapt_file = $ARGV[7];
   	$noadapt = $ARGV[8];
}
else
{
	$syst = 'win';
	$fas_file = 'Ea_tag_MID1.tag';
	$fas_folder= 'd:/pipe_clean/QDD2/1307712142/Ea_tag_1307712142';
	$outfolder = 'temp1';
#	$blast_path = 'c:/BLAST2_2_18/bin/';
  $blast_path = 'c:/Program files/NCBI/blast-2.2.25+/bin/';
  $blast_version = 'blast+';
	$Evector = '1E-3';
  $adapt_file = 'adapter.fas'; # with blast+ db file must be in the same folder as cut_vector.pl
#  $adapt_folder = 'd:/pipe_clean/QDD2';
  $noadapt = 1;
  print "Direct parameteres are used in cut_vector.pl\n";
}

# if $dist == 1 produce length distridution data
my $dist = 1;

my %bl_beg = ();
my %bl_end = ();
my %seq = ();
my %adapt = ();


	my $outfile = $fas_file;
	$outfile =~ s/\..+$/\.wov/;
	$outfile = $outfolder.'/'.$outfile;
	my $out_blast = $fas_file;
	$out_blast =~ s/\..+$/_m8\.tbl/;
	$out_blast = $outfolder.'/'.$out_blast;
	my $out_length = $fas_file;
	$out_length =~ s/\..+$/_length\.tbl/;
	$out_length = $outfolder.'/'.$out_length;

  $fas_file = $fas_folder.'/'.$fas_file;
#  $adapt_file = $adapt_folder.'/'.$adapt_file;


# blast sequence file againts the adaptor file; filter is switched off, *
#to be able to clean even low complexity sequences
call_blast_adapter_no_filter($fas_file, $out_blast, $adapt_file, $Evector, $blast_path, $blast_version, $syst);

%seq = read_hach_from_fasta_code_wo_space($fas_file);
my %seq_l = length_sequences(\%seq);

open_file_in($out_blast, 'BL');
while(my $line = <BL>)
{
	my $code = '';
	my @line = split(' ', $line);
	if ($line[6] < $seq_l{$line[0]} - $line[7])
	{
		if (exists $bl_beg{$line[0]})
		{
			if ($line[7]> $bl_beg{$line[0]})
			{
				$bl_beg{$line[0]} = $line[7];
			}
		}
		else
		{
			$bl_beg{$line[0]} = $line[7];
		}
	}
	else
	{
		if (exists $bl_end{$line[0]})
		{
			if ($line[6] < $bl_end{$line[0]})
			{
				$bl_end{$line[0]} = $line[6];
			}
		}
		else
		{
			$bl_end{$line[0]} = $line[6];
		}
	}
	
}
close BL;

foreach my $end (keys %bl_end)
{
	my $seq_t = substr($seq{$end}, 0, $bl_end{$end}-1);
	$seq{$end} = $seq_t;
}
foreach my $beg (keys %bl_beg)
{
	my $l = length $seq{$beg};
	if ($l - $bl_beg{$beg}> 0)
	{
		my $seq_t = substr($seq{$beg}, $bl_beg{$beg}, $l - $bl_beg{$beg});
		$seq{$beg} = $seq_t;
	}
	else
	{
		$seq{$beg} = '';
	}
}

my $out_noadapt = $outfile;
$out_noadapt =~ s/\.wov/\.woa/;
if ($noadapt ==1)
{
	open_file_out($out_noadapt, 'NOA');
}
open_file_out($outfile, 'FAS');
open_file_out($out_length, 'OUT');
print OUT "code;orig_length;bases_cut_beg;bases_cut_end;length_wov;\n";
my %orig = ();
my %cut = ();
my %diff =();
my $step = 10;

foreach my $seq (sort keys %seq)
{
	my $int = int($seq_l{$seq}/$step)*$step;
	++$orig{$int};
	if (exists $bl_beg{$seq})
	{
		print FAS ">$seq\n$seq{$seq}\n";
		print OUT "$seq;$seq_l{$seq};";
		print OUT "$bl_beg{$seq};";
	}
	elsif ($noadapt ==1)
	{
		print NOA ">$seq\n$seq{$seq}\n";
		print OUT "$seq;$seq_l{$seq};";
		print OUT "0;";
	}
	else
	{
		print FAS ">$seq\n$seq{$seq}\n";
		print OUT "$seq;$seq_l{$seq};";
		print OUT "0;";		
	}

	if (exists $bl_end{$seq})
	{
		my $t = $seq_l{$seq} - $bl_end{$seq} +1;
		print OUT "$t;";
	}
	else
	{
		print OUT "0;";
	}
	my $l = length $seq{$seq};
	$int = int($l/$step)*$step;
	++$cut{$int};
	$int = int(($seq_l{$seq}-$l)/$step)*$step;
	++$diff{$int};
	print OUT "$l;\n";

}
close OUT;
close FAS; 	
if ($noadapt == 1)
{
	close NOA;
}


exit;

